@@ -6,6 +6,7 @@
  * Copyright (C) 2005-2008 Linus Walleij <triad@df.lth.se>
  * Copyright (C) 2006-2007 Marcus Meissner
  * Copyright (C) 2007 Ted Bullock
+ * Copyright (C) 2008 Chris Bagwell <chris@cnpbagwell.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,14 +43,17 @@
 
 #include "ptp-pack.c"
 
-/* To enable debug prints, switch on this */
+/* To enable debug prints for USB stuff, switch on this */
 //#define ENABLE_USB_BULK_DEBUG
 
-/* this must not be too short - the original 4000 was not long
-   enough for big file transfers. I imagine the player spends a 
-   bit of time gearing up to receiving lots of data. This also makes
-   connecting/disconnecting more reliable */
-#define USB_TIMEOUT		10000
+/* Default USB timeout length.  This can be overridden as needed
+ * but should start with a reasonable value so most common 
+ * requests can be completed.  The original value of 4000 was
+ * not long enough for large file transfer.  Also, players can
+ * spend a bit of time collecting data.  Higher values also
+ * make connecting/disconnecting more reliable.
+ */
+#define USB_TIMEOUT_DEFAULT     10000
 
 /* USB control message data phase direction */
 #ifndef USB_DP_HTD
@@ -247,6 +251,25 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
           usb_close(devh);
           return 1;
         }
+#ifdef LIBUSB_HAS_GET_DRIVER_NP
+	{
+	  /*
+	   * Specifically avoid probing anything else than USB mass storage devices
+	   * and non-associated drivers in Linux.
+	   */
+	  char devname[0x10];
+  
+	  devname[0] = '\0';
+	  ret = usb_get_driver_np(devh,
+				  dev->config[i].interface[j].altsetting[k].iInterface,
+				  devname,
+				  sizeof(devname));
+	  if (devname[0] != '\0' && strcmp(devname, "usb-storage")) {
+	    printf("avoid probing device using kernel interface \"%s\"\n", devname);
+	    return 0;
+	  }
+	}
+#endif
       }
     }
   }
@@ -282,7 +305,7 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
 			 4,
 			 (char *) buf,
 			 sizeof(buf),
-			 USB_TIMEOUT);
+                         USB_TIMEOUT_DEFAULT);
 
   // Dump it, if requested
   if (dumpfile != NULL && ret > 0) {
@@ -315,7 +338,7 @@ static int probe_device_descriptor(struct usb_device *dev, FILE *dumpfile)
 			 5,
 			 (char *) buf,
 			 sizeof(buf),
-			 USB_TIMEOUT);
+                         USB_TIMEOUT_DEFAULT);
 
   // Dump it, if requested
   if (dumpfile != NULL && ret > 0) {
@@ -586,7 +609,7 @@ char const * const get_playlist_extension(PTP_USB *ptp_usb)
 }
 
 static void
-ptp_debug (PTPParams *params, const char *format, ...)
+libusb_glue_debug (PTPParams *params, const char *format, ...)
 {  
         va_list args;
 
@@ -603,7 +626,7 @@ ptp_debug (PTPParams *params, const char *format, ...)
 }  
 
 static void
-ptp_error (PTPParams *params, const char *format, ...)
+libusb_glue_error (PTPParams *params, const char *format, ...)
 {  
         va_list args;
 
@@ -690,7 +713,7 @@ ptp_read_func (
 #ifdef ENABLE_USB_BULK_DEBUG
     printf("Reading in 0x%04lx bytes\n", toread);
 #endif
-    result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, (char*)bytes, toread, USB_TIMEOUT);
+    result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, (char*)bytes, toread, ptp_usb->timeout);
 #ifdef ENABLE_USB_BULK_DEBUG
     printf("Result of read: 0x%04x\n", result);
 #endif
@@ -755,16 +778,12 @@ ptp_read_func (
     printf("<==USB IN\n");
     printf("Zero Read\n");
 #endif
-    zeroresult = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &temp, 0, USB_TIMEOUT);
+    zeroresult = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &temp, 0, ptp_usb->timeout);
     if (zeroresult != 0)
       printf("LIBMTP panic: unable to read in zero packet, response 0x%04x", zeroresult);
   }
   
-  if (result > 0) {
-    return (PTP_RC_OK);
-  } else {
-    return PTP_ERROR_IO;
-  }
+  return PTP_RC_OK;
 }
 
 static short
@@ -786,6 +805,7 @@ ptp_write_func (
     return PTP_ERROR_IO;
   }
   while (curwrite < size) {
+    unsigned long usbwritten = 0;
     towrite = size-curwrite;
     if (towrite > CONTEXT_BLOCK_SIZE) {
       towrite = CONTEXT_BLOCK_SIZE;
@@ -796,18 +816,21 @@ ptp_write_func (
       }
     }
     handler->getfunc(NULL, handler->private,towrite,bytes,&towrite);
-    result = USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char*)bytes,towrite,USB_TIMEOUT);
+    while (usbwritten < towrite) {
+	    result = USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,((char*)bytes+usbwritten),towrite-usbwritten,ptp_usb->timeout);
 #ifdef ENABLE_USB_BULK_DEBUG
-    printf("USB OUT==>\n");
-    data_dump_ascii (stdout,bytes,towrite,16);
+	    printf("USB OUT==>\n");
+	    data_dump_ascii (stdout,bytes+usbwritten,result,16);
 #endif
-    if (result < 0) {
-      return PTP_ERROR_IO;
+	    if (result < 0) {
+	      return PTP_ERROR_IO;
+	    }
+	    // check for result == 0 perhaps too.
+	    // Increase counters
+	    ptp_usb->current_transfer_complete += result;
+	    curwrite += result;
+	    usbwritten += result;
     }
-    // Increase counters
-    ptp_usb->current_transfer_complete += result;
-    curwrite += result;
-
     // call callback
     if (ptp_usb->callback_active) {
       if (ptp_usb->current_transfer_complete >= ptp_usb->current_transfer_total) {
@@ -841,7 +864,7 @@ ptp_write_func (
       printf("USB OUT==>\n");
       printf("Zero Write\n");
 #endif
-      result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)"x",0,USB_TIMEOUT);
+      result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)"x",0,ptp_usb->timeout);
     }
   }
     
@@ -985,7 +1008,7 @@ ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
 		ret = PTP_ERROR_IO;
 	}
 	if (written != towrite && ret != PTP_ERROR_CANCEL && ret != PTP_ERROR_IO) {
-		ptp_error (params, 
+		libusb_glue_error (params, 
 			"PTP: request code 0x%04x sending req wrote only %ld bytes instead of %d",
 			req->Code, written, towrite
 		);
@@ -1111,7 +1134,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		}
 		if (dtoh16(usbdata.code)!=ptp->Code) {
 			if (FLAG_IGNORE_HEADER_ERRORS(ptp_usb)) {
-				ptp_debug (params, "ptp2/ptp_usb_getdata: detected a broken "
+				libusb_glue_debug (params, "ptp2/ptp_usb_getdata: detected a broken "
 					   "PTP header, code field insane, expect problems! (But continuing)");
 				// Repair the header, so it won't wreak more havoc, don't just ignore it.
 				// Typically these two fields will be broken.
@@ -1125,7 +1148,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 				// getting data. It appears Windows ignores the contents of this 
 				// field entirely.
 				if (ret < PTP_RC_Undefined || ret > PTP_RC_SpecificationOfDestinationUnsupported) {
-					ptp_debug (params, "ptp2/ptp_usb_getdata: detected a broken "
+					libusb_glue_debug (params, "ptp2/ptp_usb_getdata: detected a broken "
 						   "PTP header, code field insane.");
 					ret = PTP_ERROR_IO;
 				}
@@ -1133,10 +1156,15 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 			}
 		}
 		if (usbdata.length == 0xffffffffU) {
+			/* Copy first part of data to 'data' */
+			handler->putfunc(
+				params, handler->private, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
+				&written
+			);
 			/* stuff data directly to passed data handler */
 			while (1) {
 				unsigned long readdata;
-				int xret;
+				uint16_t xret;
 
 				xret = ptp_read_func(
 					PTP_USB_BULK_HS_MAX_PACKET_LEN_READ,
@@ -1146,7 +1174,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 					0
 				);
 				if (xret != PTP_RC_OK)
-					return ret;
+					return xret;
 				if (readdata < PTP_USB_BULK_HS_MAX_PACKET_LEN_READ)
 					break;
 			}
@@ -1176,7 +1204,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 			/* Ignore reading one extra byte if device flags have been set */
 			} else if(!FLAG_NO_ZERO_READS(ptp_usb) &&
 				  (rlen - dtoh32(usbdata.length) == 1)) {
-			  ptp_debug (params, "ptp2/ptp_usb_getdata: read %d bytes "
+			  libusb_glue_debug (params, "ptp2/ptp_usb_getdata: read %d bytes "
 				     "too much, expect problems!", 
 				     rlen - dtoh32(usbdata.length));
 			}
@@ -1208,7 +1236,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		  // need to read in extra byte and discard it
 		  int result = 0;
 		  char byte = 0;
-		  result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &byte, 1, USB_TIMEOUT);
+                  result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &byte, 1, ptp_usb->timeout);
 		  
 		  if (result != 1)
 		    printf("Could not read in extra byte for PTP_USB_BULK_HS_MAX_PACKET_LEN_READ long file, return value 0x%04x\n", result);
@@ -1219,7 +1247,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 #ifdef ENABLE_USB_BULK_DEBUG
 		  printf("Reading in zero packet after header\n");
 #endif
-		  zeroresult = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &zerobyte, 0, USB_TIMEOUT);
+                  zeroresult = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, &zerobyte, 0, ptp_usb->timeout);
 		  
 		  if (zeroresult != 0)
 		    printf("LIBMTP panic: unable to read in zero packet, response 0x%04x", zeroresult);
@@ -1261,7 +1289,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	// after a NULL packet when it should send the response. This code ignores
 	// such illegal packets.
 	while (ret==PTP_RC_OK && rlen<PTP_USB_BULK_HDR_LEN && usbresp.length==0) {
-	  ptp_debug (params, "ptp_usb_getresp: detected short response "
+	  libusb_glue_debug (params, "ptp_usb_getresp: detected short response "
 		     "of %d bytes, expect problems! (re-reading "
 		     "response), rlen");
 	  ret = ptp_usb_getpacket(params, &usbresp, &rlen);
@@ -1280,7 +1308,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	printf("%04x\n", ret);
 #endif
 	if (ret!=PTP_RC_OK) {
-/*		ptp_error (params,
+/*		libusb_glue_error (params,
 		"PTP: request code 0x%04x getting resp error 0x%04x",
 			resp->Code, ret);*/
 		return ret;
@@ -1291,7 +1319,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	resp->Transaction_ID=dtoh32(usbresp.trans_id);
 	if (FLAG_IGNORE_HEADER_ERRORS(ptp_usb)) {
 		if (resp->Transaction_ID != params->transaction_id-1) {
-			ptp_debug (params, "ptp_usb_getresp: detected a broken "
+			libusb_glue_debug (params, "ptp_usb_getresp: detected a broken "
 				   "PTP header, transaction ID insane, expect "
 				   "problems! (But continuing)");
 			// Repair the header, so it won't wreak more havoc.
@@ -1328,15 +1356,15 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 	ret = PTP_RC_OK;
 	switch(wait) {
 	case PTP_EVENT_CHECK:
-		result=USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *)&usbevent,sizeof(usbevent),USB_TIMEOUT);
+                result=USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *)&usbevent,sizeof(usbevent),ptp_usb->timeout);
 		if (result==0)
-			result = USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *) &usbevent, sizeof(usbevent), USB_TIMEOUT);
+                        result = USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *) &usbevent, sizeof(usbevent), ptp_usb->timeout);
 		if (result < 0) ret = PTP_ERROR_IO;
 		break;
 	case PTP_EVENT_CHECK_FAST:
-		result=USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *)&usbevent,sizeof(usbevent),USB_TIMEOUT);
+                result=USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *)&usbevent,sizeof(usbevent),ptp_usb->timeout);
 		if (result==0)
-			result = USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *) &usbevent, sizeof(usbevent), USB_TIMEOUT);
+                        result = USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *) &usbevent, sizeof(usbevent), ptp_usb->timeout);
 		if (result < 0) ret = PTP_ERROR_IO;
 		break;
 	default:
@@ -1344,13 +1372,13 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 		break;
 	}
 	if (ret!=PTP_RC_OK) {
-		ptp_error (params,
+		libusb_glue_error (params,
 			"PTP: reading event an error 0x%04x occurred", ret);
 		return PTP_ERROR_IO;
 	}
 	rlen = result;
 	if (rlen < 8) {
-		ptp_error (params,
+		libusb_glue_error (params,
 			"PTP: reading event an short read of %ld bytes occurred", rlen);
 		return PTP_ERROR_IO;
 	}
@@ -1387,7 +1415,7 @@ ptp_usb_control_cancel_request (PTPParams *params, uint32_t transactionid) {
 	htod32a(&buffer[2],transactionid);
 	ret = usb_control_msg(ptp_usb->handle, 
 			      USB_TYPE_CLASS | USB_RECIP_INTERFACE,
-			      0x64, 0x0000, 0x0000, (char *) buffer, sizeof(buffer), USB_TIMEOUT);
+                              0x64, 0x0000, 0x0000, (char *) buffer, sizeof(buffer), ptp_usb->timeout);
 	if (ret < sizeof(buffer))
 		return PTP_ERROR_IO;
 	return PTP_RC_OK;
@@ -1397,8 +1425,6 @@ static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device*
 {
   usb_dev_handle *device_handle;
   
-  params->error_func=NULL;
-  params->debug_func=NULL;
   params->sendreq_func=ptp_usb_sendreq;
   params->senddata_func=ptp_usb_senddata;
   params->getresp_func=ptp_usb_getresp;
@@ -1412,6 +1438,8 @@ static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device*
    */
   params->byteorder = PTP_DL_LE;
   
+  ptp_usb->timeout = USB_TIMEOUT_DEFAULT;
+  
   if ((device_handle = usb_open(dev))){
     if (!device_handle) {
       perror("usb_open()");
@@ -1710,17 +1738,27 @@ void close_device (PTP_USB *ptp_usb, PTPParams *params)
   close_usb(ptp_usb);
 }
 
+void set_usb_device_timeout(PTP_USB *ptp_usb, int timeout)
+{
+    ptp_usb->timeout = timeout;
+}
+
+void get_usb_device_timeout(PTP_USB *ptp_usb, int *timeout)
+{
+    *timeout = ptp_usb->timeout;
+}
+
 static int usb_clear_stall_feature(PTP_USB* ptp_usb, int ep)
 {
   
   return (usb_control_msg(ptp_usb->handle,
 			  USB_RECIP_ENDPOINT, USB_REQ_CLEAR_FEATURE, USB_FEATURE_HALT,
-			  ep, NULL, 0, USB_TIMEOUT));
+                          ep, NULL, 0, ptp_usb->timeout));
 }
 
 static int usb_get_endpoint_status(PTP_USB* ptp_usb, int ep, uint16_t* status)
 {
   return (usb_control_msg(ptp_usb->handle,
 			  USB_DP_DTH|USB_RECIP_ENDPOINT, USB_REQ_GET_STATUS,
-			  USB_FEATURE_HALT, ep, (char *)status, 2, USB_TIMEOUT));
+                          USB_FEATURE_HALT, ep, (char *)status, 2, ptp_usb->timeout));
 }
